// Copyright 2019 the Go-FUSE Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

package fs

import "github.com/hanwen/go-fuse/v2/fuse"

func setBlocks(out *fuse.Attr) {
}
